import { AssetManager, assetManager, Color, Layers, Node, sp, UITransformComponent } from "cc";




export class SpineAnimation {
    private __node: Node | null;
    private __skeleton: sp.Skeleton | null;
    private __url: { url: string, bundle: string } | null = null;

    constructor(node: string | Node, cacheMode: number = 1, enableBatch: boolean = true) {
        if (node instanceof Node) {
            this.__node = node;
        } else {
            this.__node = new Node(node);
            this.__node.layer = Layers.Enum.UI_2D;
            let ui: UITransformComponent = this.__node.addComponent(UITransformComponent);
            ui.setContentSize(100, 100);
            ui.setAnchorPoint(0.5, 0.5);
        }
        this.__skeleton = this.__node.getComponent(sp.Skeleton);
        if (this.__skeleton == null) {
            this.__skeleton = this.__node.addComponent(sp.Skeleton);
        }
        if (cacheMode == 0) {
            this.__skeleton.setAnimationCacheMode(sp.AnimationCacheMode.REALTIME);
        } else if (cacheMode == 1) {
            this.__skeleton.setAnimationCacheMode(sp.AnimationCacheMode.SHARED_CACHE);
        } else {
            this.__skeleton.setAnimationCacheMode(sp.AnimationCacheMode.PRIVATE_CACHE);
        }
        // this.__skeleton.premultipliedAlpha = false;
        this.__skeleton.enableBatch = false;
    }

    set url(value: { url: string, bundle: string } | null) {
        this.__url = value;
        this.__urlChanged();
    }

    get url(): { url: string, bundle: string } | null {
        return this.__url!;
    }

    private __urlChanged(): void {
        let bundle: AssetManager.Bundle | null = assetManager.getBundle(this.url!.bundle);
        if (bundle) {
            let res: sp.SkeletonData | null = bundle.get(this.url!.url);
            if (res) {
                this.__assetLoaded(null, res);
            } else {
                bundle.load(this.url!.url, sp.SkeletonData, this.__assetLoaded.bind(this));
            }
        } else {
            assetManager.loadBundle(this.url!.bundle, null, this.__bundleLoaded.bind(this));
        }
    }

    private __bundleLoaded(err: Error | null, bundle: AssetManager.Bundle): void {
        if (err) {
            throw new Error(err.message);
        }
        let res: sp.SkeletonData | null = bundle.get(this.url!.url);
        if (res) {
            this.__assetLoaded(null, res);
        } else {
            bundle.load(this.url!.url, sp.SkeletonData, this.__assetLoaded.bind(this));
        }
    }

    private __assetLoaded(err: Error | null, skData: sp.SkeletonData): void {
        if (err) {
            throw new Error(err.message);
        }
        this.__skeleton!.skeletonData = skData;
        this.__skeleton!.setAnimation(0, this.randomAnimation, true);
    }


    playRandomAnimation(): void {
        this.__skeleton!.setAnimation(0, this.randomAnimation, true);
    }

    get boneCount(): number {
        let skData: any = this.__skeleton!.skeletonData;
        if (!skData) {
            return 0;
        }
        return skData["_skeletonCache"]["bones"].length;
    }

    get randomAnimation(): string {
        let skData: any = this.__skeleton!.skeletonData;
        let list = skData["_skeletonCache"]["animations"]
        let index: number = Math.ceil(Math.random() * (list.length - 1));
        let ani = list[index];
        return ani.name;
    }

    get node(): Node {
        return this.__node!;
    }

    get skeleton(): sp.Skeleton {
        return this.__skeleton!;
    }

    destroy(): void {
        this.__skeleton!.destroy();
        this.__node!.destroy();
        this.__node = null;
    }
}